require 'emcee/action'

class Emcee
  class Agent
    attr_reader :runner, :name, :version, :description, :nodes

    def initialize(runner, agent_data)
      @runner = runner
      @name = agent_data[:agent]
      @version = agent_data[:version]
      @description = agent_data[:description]
      @nodes = []
    end

    def matches?(other)
      if other.respond_to?(:name)
        return (self.name == other.name and self.version == other.version)
      end
      return false if other[:agent] and name != other[:agent]
      return false if other[:version] and version != other[:version]
      return other[:agent] || other[:version]
    end

    def actions
      @actions ||= action_names.collect {|action_name| Emcee::Action.new(self, action_interface(action_name)) }
    end

    def action_interface(action_name)
      ddl.action_interface(action_name)
    end

    def action_names
      ddl ? ddl.actions : []
    end

    def action_by_name(name)
      actions.find {|a| a.name == name}
    end

    def ddl
      runner.ddl(name)
    end

    def has_node?(node)
      nodes.detect {|n| n.matches?(node) }
    end

    def add_node(node)
      nodes << node unless has_node?(node)
    end

    def inspect
      YAML.dump(self)
    end

    def to_json(*a)
      { 'name'        => name,
        'version'     => version,
        'description' => description,
        'nodes'       => nodes.collect { |n| n.name },
        'actions'     => actions.map do |a|
          { 'name'        => a.name,
            'description' => a.description,
            'input'       => a.input
          }
        end
      }.to_json
    end
  end
end
