require 'spec_helper'

describe Baseline do
  describe ".create_from_report!" do
    it "should create a Baseline from an inspect report" do
      report = Factory(:inspect_report)

      Baseline.create_from_report!(report)

      Baseline.count.should == 1
    end

    it "should create corresponding baseline resources for the report resource statuses" do
      report = Report.create_from_yaml(File.read(File.join(File.dirname(__FILE__), '../fixtures/reports/inspect.yaml')))

      baseline = Baseline.create_from_report!(report)

      baseline.resources.map do |res|
        [res.resource_type, res.title]
      end.should =~ report.resource_statuses.map{ |res| [res.resource_type, res.title] }

      baseline.resources.map(&:properties).flatten.map do |prop|
        [prop.name, prop.value.to_s]
      end.should =~ report.events.map{ |ev| [ev.property, ev.previous_value.to_s] }
    end

    it "should not create baseline resources for failed inspect report resources" do
      report = Factory(:inspect_report)
      good   = Factory(:successful_resource, :title => "good", :report => report)
      evil   = Factory(:failed_resource, :title => "evil", :report => report)

      # Now, create a baseline from that, and it should have only one
      # property, because we transfer only non-failed things.
      baseline = Baseline.create_from_report!(report)
      baseline.resources.count.should == 1
      baseline.resources.first.title.should == "good"
    end

    it "should reject non-inspect reports" do
      report = Factory(:successful_report)

      expect { Baseline.create_from_report!(report) }.to raise_error(IncorrectReportKind, /expected.*inspect/)
    end
  end
end
